package sun.security.krb5.internal.ktab;

import sun.security.krb5.internal.KerberosTime;
import sun.security.krb5.RealmException;
import sun.security.krb5.PrincipalName;
import sun.security.krb5.Realm;
import java.io.IOException;
import sun.security.krb5.internal.Krb5;
import java.io.InputStream;
import sun.security.krb5.internal.util.KrbDataInputStream;

public class KeyTabInputStream extends KrbDataInputStream implements KeyTabConstants
{
    boolean DEBUG;
    int index;
    
    public KeyTabInputStream(InputStream is) {
        super(is);
        this.DEBUG = Krb5.DEBUG;
    }
    
    int readEntryLength() throws IOException {
        return this.read(4);
    }
    
    KeyTabEntry readEntry(int index, int n) throws IOException, RealmException {
        this.index = index;
        if (this.index == 0) {
            return null;
        }
        if (this.index < 0) {
            this.skip((long)Math.abs(this.index));
            return null;
        }
        int read = this.read(2);
        this.index -= 2;
        if (n == 1281) {
            --read;
        }
        Realm nameRealm = new Realm(this.readName());
        String[] nameStrings = new String[read];
        for (int i = 0; i < read; ++i) {
            nameStrings[i] = this.readName();
        }
        int read2 = this.read(4);
        this.index -= 4;
        PrincipalName principalName = new PrincipalName(read2, nameStrings, nameRealm);
        KerberosTime timeStamp = this.readTimeStamp();
        int n2 = this.read() & 0xFF;
        --this.index;
        int read3 = this.read(2);
        this.index -= 2;
        int read4 = this.read(2);
        this.index -= 2;
        byte[] key = this.readKey(read4);
        this.index -= read4;
        if (this.index >= 4) {
            int read5 = this.read(4);
            if (read5 != 0) {
                n2 = read5;
            }
            this.index -= 4;
        }
        if (this.index < 0) {
            throw new RealmException("Keytab is corrupted");
        }
        this.skip((long)this.index);
        return new KeyTabEntry(principalName, nameRealm, timeStamp, n2, read3, key);
    }
    
    byte[] readKey(int len) throws IOException {
        byte[] b = new byte[len];
        this.read(b, 0, len);
        return b;
    }
    
    KerberosTime readTimeStamp() throws IOException {
        this.index -= 4;
        return new KerberosTime((long)this.read(4) * 1000L);
    }
    
    String readName() throws IOException {
        int read = this.read(2);
        this.index -= 2;
        byte[] array = new byte[read];
        this.read(array, 0, read);
        this.index -= read;
        String str = new String(array);
        if (this.DEBUG) {
            System.out.println(">>> KeyTabInputStream, readName(): " + str);
            return str;
        }
        return str;
    }
}
